library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity riscv_example is
	port (
		clk25			: in		std_logic;
		reset_n		: in		std_logic;
		led_red		: out		std_logic;
		led_green	: out		std_logic;
		gpio			: inout	std_logic_vector(7 downto 0);
		uart_rx		: in		std_logic;
		uart_tx		: out		std_logic
	);
end entity riscv_example;



architecture riscv of riscv_example is

	signal counter : integer range 1 to 12500000 := 1;
	signal led_internal : std_logic := '0';

	component qsys is
		port (
			clk_clk       : in    std_logic                    := 'X';
			pio_export    : inout std_logic_vector(7 downto 0) := (others => 'X');
			reset_reset_n : in    std_logic                    := 'X';
			uart_rxd      : in    std_logic                    := 'X';
			uart_txd      : out   std_logic;
			led_export    : out   std_logic
		);
	end component qsys;

begin

	u0 : component qsys
		port map (
			clk_clk       => clk25,       --   clk.clk
			pio_export    => gpio,    --   pio.export
			reset_reset_n => reset_n,  -- reset.reset_n
			uart_rxd		  => uart_rx,
			uart_txd		  => uart_tx,
			led_export    => led_green
		);

	process (clk25)
	begin
		if (clk25 = '1' and clk25'EVENT) then
			if (counter < 12500000) then
				counter <= counter + 1;
			else
				led_internal <= not led_internal;
				counter <= 1;
			end if;
		end if;				
	end process;
	
	led_red <= led_internal;

end architecture riscv;