#include "FPGAConfig.h"
#include "hal.h"

void UARTIRQHandler(void) {
	char c = UARTGet(g_UART);
	UARTPut(g_UART, c);
	g_UART->status = 0;
}

int main(void) {
	
	// Enable receive interrupt on UART
	g_UART->control = UART_RRDY;

	// Initialize Interrupt Control
	halSetInterruptHandler(UARTIRQHandler, IRQ_UART);
	halEnableIRQ(IRQ_UART_BIT);
	halEnableInterrupts();

	// Set all GPIOs to output
    g_GPIO->direction = 0xFF;

	// Toggle LED once per second. Instruction clock (FOSC) is 25 MHz
	uint32_t ledPrevTime = get_time();
	uint32_t gpioPrevTime = get_time();

	// Loop forever.
    while (1) {
                
		uint32_t currentTime = get_time();
		
		// Toggle LED when time difference is greater than FOSC
		if (currentTime - ledPrevTime > FOSC) {
			g_LED->green ^= 1;
			ledPrevTime = currentTime;
		}
		
		// Increment GPIO every 100 ms.
		if (currentTime - gpioPrevTime > FOSC / 10) {
		g_GPIO->gpio++;
			gpioPrevTime = currentTime;
		}
	
    }

	// This line should never be executed.
    return 0;

}