#include "uart.h"

void UARTWriteInt(UART* pUART, int32_t i, bool newline) {
	
	// An 32 bit integer has at most 10 digits + sign + zero byte
	char buffer[12] = {0};

	// To prevent overflow, handle this as special case
	if (i == INT32_MIN) {

		buffer[0] = '-';
		buffer[1] = '2';
		buffer[2] = '1';
		buffer[3] = '4';
		buffer[4] = '7';
		buffer[5] = '4';
		buffer[6] = '8';
		buffer[7] = '3';
		buffer[8] = '6';
		buffer[9] = '4';
		buffer[10] = '8';
		buffer[11] = 0;

	} else {

		uint8_t bufferIndex = 0;

		if (i < 0) {
			i = -i;
			buffer[bufferIndex++] = '-';
		}

		int32_t temp = i / 10;

		while (temp) {
			temp /= 10;
			bufferIndex++;
		}

		do {
			buffer[bufferIndex--] = (i % 10) + '0';
			i /= 10;
		} while(i);

	}

	UARTWrite(pUART, buffer);
	
	if (newline) {
		UARTWrite(pUART, "\n");
	}
	
}

void UARTWriteHex32(UART* pUART, uint32_t ui, bool newline) {
	UARTWriteHex8(pUART, (ui>>24) & 0xFF, false);
	UARTWriteHex8(pUART, (ui>>16) & 0xFF, false);
	UARTWriteHex8(pUART, (ui>>8) & 0xFF, false);
	UARTWriteHex8(pUART, (ui) & 0xFF, newline);
}

void UARTWriteHex8(UART* pUART, uint8_t byte, bool newline) {
	uint8_t l = byte & 0x0F;
	uint8_t h = (byte >> 4) & 0x0F;
	l = (l < 0x0A) ? l + '0' : l + 'A' - 10;
	h = (h < 0x0A) ? h + '0' : h + 'A' - 10;
	UARTPut(pUART, h);
	UARTPut(pUART, l);
	if (newline) {
		UARTWrite(pUART, "\n");
	}
}

void UARTWrite(UART* pUART, const char* str) {
	while (*str) {
		if (*str == '\n') UARTPut(pUART, '\r');
		UARTPut(pUART, *str++);
	}
}

void UARTPut(UART* pUART, char c) {
	while (!(pUART->status & UART_TRDY));
	pUART->txdata = c;
}

char UARTGet(UART* pUART) {
	while (!(pUART->status & UART_RRDY));  
	return pUART->rxdata;
}

bool UARTGetNonBlocking(UART* pUART, char* c) {
	if (pUART->status & UART_RRDY) {
		*c = pUART->rxdata;
		return true;
	}
	return false;
}