#ifndef FPGACONFIG_H
#define FPGACONFIG_H

#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>

#include "riscvdef.h"

#include "uart.h"
#include "timer.h"

// Configure whether heap is placed on OCRAM or SDRAM
#define HEAP_OCRAM 1
#define HEAP_SDRAM 2
#define HEAP_CONFIG HEAP_OCRAM

// Frequency of ORCA Core
#define FOSC 25000000UL

#define MEMADDR_TIMER	0x00010100
#define MEMADDR_SITIMER	0x00010200
#define MEMADDR_UART 	0x00010300
#define MEMADDR_GPIO	0x00010400
#define MEMADDR_LED		0x00010500

#define IRQ_UART 	0
#define IRQ_TIMER	30
#define IRQ_SOFT	31

#define IRQ_UART_BIT	(1 << IRQ_UART) 
#define IRQ_TIMER_BIT	(1 << IRQ_TIMER)
#define IRQ_SOFT_BIT	(1 << IRQ_SOFT)

typedef volatile struct {
	uint32_t gpio : 8;
	uint32_t : 24;
	uint32_t direction : 8;
	uint32_t : 24;
} GPIO;

typedef volatile struct {
	uint8_t green : 1;
} LED;

extern Timer* g_Timer;
extern Timer* g_SITimer;
extern GPIO* g_GPIO;
extern UART* g_UART;
extern LED* g_LED;

#endif /* FPGACONFIG_H */
