#include "hal.h"

static void_func halInterruptHandlers[30] = {0};

void halEnableInterrupts(void) {
	set_csr(mstatus, MSTATUS_MIE);
}

void halDisableInterrupts(void) {
	clear_csr(mstatus, MSTATUS_MPIE);
	clear_csr(mstatus, MSTATUS_MIE);
}


void halRaiseSoftInterrupt(void) {
	TimerSetPeriod(g_SITimer, 1);
	TimerRearm(g_SITimer);
}

void halClearSoftInterupt(void) {
	TimerDisableInterrupt(g_SITimer);
}

void halSetInterruptHandler(void_func f, uint32_t index) {
	if (index < 30) {
		halInterruptHandlers[index] = f;
	}
}

void_func halGetInterruptHandler(uint32_t index) {
	if (index < 30) {
		return halInterruptHandlers[index];
	}
	return 0x0;
}

void halEnableIRQ(uint32_t bitmask) {
		set_csr_ns(MEIMASK, bitmask);
}

void halDisableIRQ(uint32_t bitmask) {
		clear_csr_ns(MEIMASK, bitmask);
}

uintptr_t handle_trap(uintptr_t mcause, uintptr_t mepc) {
	
	// Trap exception occured - write debug information to g_UART and stop program execution.
	
	DebugWrite("Trap: ");
	DebugWriteHex32(mepc, true);
	DebugWrite("Cause: ");
	DebugWriteHex32(mcause, true);
	
	while (1) {
		// TODO: Add error recovery code.
	}
	
	return 0;
}	

uintptr_t handle_interrupt(uintptr_t mcause, uintptr_t mepc, uintptr_t stackPointer) {
	
	if (mcause & IRQ_SOFT_BIT) {
		// Interrupt by user code.
		TimerDisableInterrupt(g_SITimer);
	}

	if (mcause & IRQ_TIMER_BIT) {
		// Interrupt by periodic timer.
	}
	
	// If either soft or timer interrupt occured: Rearm timer and call scheduler.
	if (mcause & (IRQ_SOFT_BIT | IRQ_TIMER_BIT)){
		TimerRearm(g_Timer);
		// SysTick_Handler returns by MRET directly to restored context.
		SysTick_Handler(stackPointer);
		// return //
	}
	
	for (uint32_t i = 0; i < 30; ++i){
		uint32_t mask = 1 << i;
		if (mcause & mask) {
			if (halInterruptHandlers[i]) {
				halInterruptHandlers[i]();
			} else {
				// For debugging purpose; if interrupt was unhandled.
				DebugWrite("Unhandled Interrupt\n");
				DebugWrite("cause ");
				DebugWriteHex32(mcause, true);
				DebugWrite("mpec  ");
				DebugWriteHex32(mepc, true);
			}
		}
	}
	
	return mepc;
	
}